class Credentials(object):

    @property
    def accessKeyId(self):
        return self.__accessKeyId

    @property
    def secretAccessKey(self):
        return self.__secretAccessKey

    @property
    def sessionToken(self):
        return self.__sessionToken

    @accessKeyId.setter
    def accessKeyId(self, value):
        self.__accessKeyId = value

    @secretAccessKey.setter
    def secretAccessKey(self, value):
        self.__secretAccessKey = value

    @sessionToken.setter
    def sessionToken(self, value):
        self.__sessionToken = value

    def __init__(self, accessKeyId = None, secretAccessKey = None, sessionToken = None):
        self.__accessKeyId = accessKeyId
        self.__secretAccessKey = secretAccessKey
        self.__sessionToken = sessionToken



import os

from configparser import ConfigParser

from awsgen.models.profile import Profile

class ConfigurationApp(object):

    def save(self, configuration):
        self.__writeConfiguration(configuration)


    def update(self, configuration):
        self.__updateConfiguration(configuration)


    def __writeConfiguration(self, configuration):
        filename = os.path.expanduser('~/.aws/config')
        dirname = os.path.dirname(filename)

        if not os.path.exists(dirname):
            os.makedirs(dirname)

        config = ConfigParser()
        config.read(filename)

        profile = 'profile ' + configuration.profile

        if not config.has_section(profile):
            config.add_section(profile)

        config.set(profile, 'region', configuration.region)
        config.set(profile, 'output', configuration.output)

        with open(filename, 'w') as fp:
            config.write(fp)


    def __updateConfiguration(self, configuration):
        filename = os.path.expanduser('~/.aws/config')
        dirname = os.path.dirname(filename)

        if not os.path.exists(dirname):
            os.makedirs(dirname)

        config = ConfigParser()
        config.read(filename)

        profile = 'profile ' + configuration.profile

        if config.has_section(profile):
            if configuration.region:
                config.set(profile, 'region', configuration.region)
            if configuration.output:
                config.set(profile, 'output', configuration.output)

        with open(filename, 'w') as fp:
            config.write(fp)



import logging

from awsgen.applications.profile import ProfileApp

class SetActiveProfile(object):

    log = logging.getLogger(__name__)

    def __init__(self, parser=None):
        parser.add_argument('--profile', metavar='profile', required=True, dest='profile')

        
    def action(self, args):
        ProfileApp().setActive(args.profile)



import logging

from awsgen.applications.profile import ProfileApp

class GetActiveProfile(object):

    log = logging.getLogger(__name__)

    def __init__(self, parser=None):
        pass
        
    def action(self, args):
        print('[' + ProfileApp().getActive() + ']')



class Configuration(object):

    @property
    def profile(self):
        return self.__profile

    @property
    def region(self):
        return self.__region

    @property
    def output(self):
        return self.__output        

    @profile.setter
    def profile(self, value):
        self.__profile = value

    @region.setter
    def region(self, value):
        self.__region = value

    @output.setter
    def output(self, value):
        self.__output = value

    def __init__(self, profile = None, region = None, output = None):
        self.__profile = profile
        self.__region = region
        self.__output = output


class Profile(object):

    @property
    def name(self):
        return self.__name

    @property
    def trustRoleArn(self):
        return self.__trustRoleArn

    @property
    def sourceProfile(self):
        return self.__sourceProfile

    @property
    def credentials(self):
        return self.__credentials

    @name.setter
    def name(self, value):
        self.name = value

    @trustRoleArn.setter
    def trustRoleArn(self, value):
        self.__trustRoleArn = value

    @sourceProfile.setter
    def sourceProfile(self, value):
        self.__sourceProfile = value        

    @credentials.setter
    def credentials(self, value):
        self.__credentials = value

    def __init__(self, name = None, trustRoleArn = None, sourceProfile = None, credentials = None):
        self.__name = name
        self.__trustRoleArn = trustRoleArn
        self.__sourceProfile = sourceProfile
        self.__credentials = credentials



import os
import sys
import boto3
import json
import requests

from configparser import ConfigParser

from botocore.exceptions import ClientError

from awsgen.models.credentials import Credentials

from awsgen.applications.configuration import ConfigurationApp
from awsgen.applications.account import AccountApp

class TemporaryApp(object):

    def key(self, request):
        account = AccountApp().get(request.sourceProfile)
        boto3.setup_default_session(profile_name=request.sourceProfile)
        sts = boto3.client('sts')
        try:
            response = sts.assume_role(RoleArn=account.trustRoleArn, RoleSessionName=request.profile)
            credentials = Credentials(
                secretAccessKey=response['Credentials']['SecretAccessKey'],
                sessionToken=response['Credentials']['SessionToken'],
                accessKeyId=response['Credentials']['AccessKeyId']
            )
            return credentials
        except ClientError:
            print('')
            print('\tThe request signature we calculated does not match the signature you provided.')
            print('\tCheck your AWS Secret Access Key and signing method on profile [' + request.profile + ']')
            print('\tCheck your Arn role [' + account.trustRoleArn + ']')
            print('')
            sys.exit(1)


    def link(self, request):
        credentials = self.key(request)

        session = json.dumps({
            'sessionId': credentials.accessKeyId,
            'sessionKey': credentials.secretAccessKey,
            'sessionToken': credentials.sessionToken
        })

        request = requests.get("https://signin.aws.amazon.com/federation",
            params={
                'Action': 'getSigninToken',
                'SessionDuration': 43200,
                'Session': session
            }
        )

        signinToken = request.json()

        console = requests.Request('GET','https://signin.aws.amazon.com/federation',
            params={
                'Action': 'login',
                'Issuer': 'trustuser',
                'Destination': 'https://console.aws.amazon.com/',
                'SigninToken': signinToken['SigninToken']
            }
        )

        preparedLink = console.prepare()

        return preparedLink.url



import logging

from awsgen.models.request import Request

from awsgen.applications.temporary import TemporaryApp

class GetLink(object):

    log = logging.getLogger(__name__)

    def __init__(self, parser):
        parser.add_argument('--account', metavar='account', required=True, dest='account')
        parser.add_argument('--profile', metavar='profile', required=True, dest='profile')


    def action(self, args):
        request = Request(
            profile = args.profile, 
            sourceProfile = args.account
        )

        print('')
        print('Magic Link: ' + TemporaryApp().link(request))
        print('')



import logging

from awsgen.models.request import Request
from awsgen.models.profile import Profile
from awsgen.models.configuration import Configuration

from awsgen.applications.profile import ProfileApp
from awsgen.applications.account import AccountApp
from awsgen.applications.configuration import ConfigurationApp

class CreateProfile(object):

    log = logging.getLogger(__name__)

    def __init__(self, parser):
        parser.add_argument('--account', required=True, dest='account')
        parser.add_argument('--profile', required=True, dest='profile')
        parser.add_argument('--region-name', required=True, dest='region')
        parser.add_argument('--output', required=True, dest='output')


    def action(self, args):
        account = AccountApp().get(args.account)

        profile = Profile(
            name = args.profile, 
            trustRoleArn = account.trustRoleArn, 
            sourceProfile = args.account
        )

        configuration = Configuration(
            profile = args.profile, 
            region = args.region, 
            output = args.output
        )

        ProfileApp().save(profile)
        ConfigurationApp().save(configuration)



import os

from configparser import ConfigParser

from awsgen.models.profile import Profile

from awsgen.applications.account import AccountApp

class ProfileApp(object):

    def save(self, profile):
        self.__writeProfile(profile)


    def setActive(self, profile):
        self.__writeActiveProfile(profile)


    def getActive(self):
        filename = os.path.expanduser('~/.aws/credentials')
        dirname = os.path.dirname(filename)

        if not os.path.exists(dirname):
            print('File: [' + filename + '] not found')
            return

        config = ConfigParser()
        config.read(filename)

        if not config.has_section('default'):
            print('Profile: [default] not found')
            return

        return config.get('default', 'active')

    def list(self):
        filename = os.path.expanduser('~/.aws/credentials')
        dirname = os.path.dirname(filename)

        if not os.path.exists(dirname):
            print('File: [' + filename + '] not found')
            return

        config = ConfigParser()
        config.read(filename)

        sections = sorted(config.sections())

        for section in sections:
            print('[' + section + ']')
            print('')


    def __writeActiveProfile(self, profile):
        filename = os.path.expanduser('~/.aws/credentials')
        dirname = os.path.dirname(filename)

        if not os.path.exists(dirname):
            print('File: [' + filename + '] not found')
            return

        config = ConfigParser()
        config.read(filename)

        if not config.has_section(profile):
            print('Profile: [' + profile + '] not found')
            return

        config.remove_section('default')
        config.add_section('default')

        for value in config.items(profile):
            config.set('default', value[0], value[1])

        config.set('default', 'active', profile)

        with open(filename, 'w') as fp:
            config.write(fp)


    def __writeProfile(self, profile):
        filename = os.path.expanduser('~/.aws/credentials')
        dirname = os.path.dirname(filename)

        if not os.path.exists(dirname):
            os.makedirs(dirname)

        config = ConfigParser()
        config.read(filename)

        if not config.has_section(profile.name):
            config.add_section(profile.name)

        if profile.trustRoleArn:
            config.set(profile.name, 'role_arn', profile.trustRoleArn)

        if profile.sourceProfile:
            config.set(profile.name, 'source_profile', profile.sourceProfile)

        if profile.credentials:
            if profile.credentials.accessKeyId:
                config.set(profile.name, 'aws_access_key_id', profile.credentials.accessKeyId)

            if profile.credentials.secretAccessKey:
                config.set(profile.name, 'aws_secret_access_key', profile.credentials.secretAccessKey)

            if profile.credentials.sessionToken:
                config.set(profile.name, 'aws_session_token', profile.credentials.sessionToken)

        with open(filename, 'w') as fp:
            config.write(fp)



import os

from configparser import ConfigParser

from awsgen.models.account import Account

class AccountApp(object):

    def save(self, account):
        self.__saveAccount(account)


    def get(self, name):
        return self.__getAccount(name)


    def __getAccount(self, name):
        filename = os.path.expanduser('~/.awsgen/config')
        dirname = os.path.dirname(filename)

        if not os.path.exists(dirname):
            os.makedirs(dirname)

        config = ConfigParser()
        config.read(filename)

        account = Account(name=name)

        if config.has_section(name):
            account.trustRoleArn = config.get(name, 'role_arn')

        return account            


    def __saveAccount(self, account):
        filename = os.path.expanduser('~/.awsgen/config')
        dirname = os.path.dirname(filename)

        if not os.path.exists(dirname):
            os.makedirs(dirname)

        config = ConfigParser()
        config.read(filename)

        if not config.has_section(account.name):
            config.add_section(account.name)

        config.set(account.name, 'role_arn', account.trustRoleArn)

        with open(filename, 'w') as fp:
            config.write(fp)



import logging

from awsgen.models.account import Account
from awsgen.models.profile import Profile
from awsgen.models.credentials import Credentials

from awsgen.applications.profile import ProfileApp
from awsgen.applications.account import AccountApp

class Configure(object):

    log = logging.getLogger(__name__)

    def __init__(self, parser):
        parser.add_argument('--account', required=True, dest='account')
        parser.add_argument('--trust-role-arn', required=True, dest='trustRoleArn')
        parser.add_argument('--access-key-id', required=True, dest='accessKeyId')
        parser.add_argument('--secret-access-key', required=True, dest='secretAccessKey')


    def action(self, args):
        credentials = Credentials(
            accessKeyId = args.accessKeyId, 
            secretAccessKey = args.secretAccessKey
        )

        profile = Profile(
            name = args.account,
            credentials = credentials
        )

        account = Account(
            name = args.account, 
            trustRoleArn = args.trustRoleArn
        )

        ProfileApp().save(profile)
        AccountApp().save(account)



class Account(object):

    @property
    def name(self):
        return self.__name

    @property
    def trustRoleArn(self):
        return self.__trustRoleArn

    @property
    def activeProfile(self):
        return self.__activeProfile

    @name.setter
    def name(self, value):
        self.name = value

    @trustRoleArn.setter
    def trustRoleArn(self, value):
        self.__trustRoleArn = value

    @activeProfile.setter
    def activeProfile(self, value):
        self.__activeProfile = value

    def __init__(self, name = None, trustRoleArn = None, activeProfile = None):
        self.__name = name
        self.__trustRoleArn = trustRoleArn
        self.__activeProfile = activeProfile



#!/usr/bin/env python

import os

from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setupOptions = dict(
    name='awsgen',
    version='0.0.14',
    install_requires=[
        'boto3>=1.9.35',
        'botocore>=1.12.16',
        'requests', 
        'configparser'
    ],
    include_package_data=True,
    scripts=[
        'bin/aws-gen', 
        'bin/aws-gen.cmd',
    ],
    provides=[],
    platforms=['Any'],
    namespace_packages=[],
    packages=find_packages(),
    license = "BSD",
    description = 'Manage AWS Security Token Service (STS)',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    url = 'https://github.com/mvallim/aws-gen-cli',
    author='Marcos Vallim',
    author_email='tischer@gmail.com'
)

setup(**setupOptions)


import sys
import argparse

from awsgen.command.configure import Configure
from awsgen.command.getlink import GetLink
from awsgen.command.getkey import GetKey
from awsgen.command.createprofile import CreateProfile
from awsgen.command.listprofiles import ListProfiles
from awsgen.command.getactiveprofile import GetActiveProfile
from awsgen.command.setactiveprofile import SetActiveProfile

class App(object):

    def __init__(self):
        self.parser = argparse.ArgumentParser(prog='aws-gen', description='Command line interface for the aws-gen package')
        self.subparsers = self.parser.add_subparsers(description='valid subcommands', title='subcommands', help='sub-commands', dest='subparser_name')


    def run(self, args):
        self.addCommand('configure', Configure)
        self.addCommand('get-link', GetLink)
        self.addCommand('get-key', GetKey)
        self.addCommand('create-profile', CreateProfile)
        self.addCommand('list-profiles', ListProfiles)
        self.addCommand('set-active-profile', SetActiveProfile)
        self.addCommand('get-active-profile', GetActiveProfile)

        if len(args) < 1:
            self.parser.print_usage()
            sys.exit(1)

        options = self.parser.parse_args(args)
        options.func(options)

    def addCommand(self, command, obj):
        parser = self.subparsers.add_parser(command)
        parser.set_defaults(func=obj(parser).action)